package info.visone.nlpext.gui;

import java.awt.Dimension;
import java.io.File;

import javax.swing.JTextPane;

import info.visone.base.Mediator;
import info.visone.base.Network;
import info.visone.collections.NetworkCollection;
import info.visone.ext.nlp.TextNetworkUtil;
import info.visone.gui.tabs.AbstractAlgorithmTaskCard;
import info.visone.gui.tabs.option.DummyVisoneOptionItem;
import info.visone.nlpext.network.CreateAttributes;
import info.visone.nlpext.network.CreationProcess;
import info.visone.nlpext.network.TextNetwork;
import info.visone.tasks.Task;

/**
 * Panel to create missing {@link TextNetworkUtil#ATTR_NAME_SENTENCE_BEGINNINGS}
 * and {@link TextNetworkUtil#ATTR_NAME_WORD_BEGINNINGS} attributes in existing
 * text networks.
 *
 * @author matthias
 */
public class CreateAttributesCard extends AbstractAlgorithmTaskCard {
    /**
     * @param mediator
     *            the mediator
     */
    public CreateAttributesCard(final Mediator mediator) {
        super("create text attributes", mediator);
    }

    @Override
    public String getApplyText() {
        return "create attributes";
    }

    @Override
    protected Task createTask(Network network, NetworkCollection collection) {
        final TextNetwork textNetw = new CreateAttributes(network, mediator);
        return new CreationProcess(new File(""), textNetw, false, mediator);
    }

    @Override
    public boolean canWorkOnCopy() {
        return false;
    }

    @Override
    protected void initializePanel() {
        final JTextPane text = new JTextPane();
        text.setContentType("text/html");
        text.setPreferredSize(new Dimension(200, 100));
        text.setEditable(false);
        text.setText("<html><p>use this algorithm if the TEXT button doesn't"
                + " work on a text network created by an old version of the"
                + " text extension. it creates the required network attributes"
                + " that old versions didn't create.</p><p>for networks created"
                + " with the current version of the text extension, this operation"
                + " isn't necessary.</p></html>");
        addDummyOptionItem(new DummyVisoneOptionItem(text, mediator));
    }
}
